/*-------------------------------------------------------------------------
 * Copyright (c) 1999-2003 Kenneth W. Sodemann (stuffle@mac.com)
 *-------------------------------------------------------------------------
 * probentry_dlg
 *
 * Synopsis:
 *   Dialog for entry of problem reports.
 *
 * $Id: probentry_dlg.c,v 1.46 2003/05/18 21:54:38 stuffle Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 59 Temple Place, Suite 330
 * Boston, MA  02111-1307  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gnome.h>
#include <gconf/gconf-client.h>
#include <libpq-fe.h>

#include "db_utils.h"
#include "defs.h"
#include "error_chks.h"
#include "gstr_utils.h"
#include "gtkutils.h"
#include "mainwin.h"
#include "util.h"

#define PK_ARRAY                "pk_array"
#define AUX_DATA_ARRAY          "aux_data_array"
/*
 *  Define the names of some of the key widgets in the dialog.  
 */
#define PRJ_MENU                "prj_option_menu"
#define VER_MENU                "ver_option_menu"
#define PT_MENU                 "pt_option_menu"
#define SVR_MENU                "svr_option_menu"
#define SUB_ENTRY               "subject_entry"
#define DESCR_TEXT              "descr_text"


/*
 * 'Null' version display data (for the combo boxes).
 */
#define NULL_NAME           _("<None>")
#define NULL_NAME_SIZE      6

/*
 *  These queries are used to fill in the selection lists for 
 *  project, problem type, and severity.
 */
#define PK_NUM_POS   0
#define NAME_POS     1
#define DEF_SEVR_POS 2
#define GET_PROJECTS "SELECT p.project_num, p.name \
                        FROM project p, submitter s \
                       WHERE p.active = TRUE \
                         AND p.project_num = s.project_num \
                         AND s.login_id = '%s'"
#define GET_PROBLEM_TYPES "SELECT type_num, name, severity_num \
                             FROM problem_type \
                            WHERE active = TRUE \
                            ORDER BY order_num "
#define GET_SEVERITIES "SELECT severity_num, name, order_num \
                          FROM severity \
                         WHERE active = TRUE \
                         ORDER BY order_num"
#define GET_LOW_STATUS "SELECT status_num \
                          FROM status \
                         WHERE active = TRUE \
                           AND order_num in \
                                (SELECT MIN (order_num) \
                                   FROM status \
                                  WHERE active = TRUE)" 
                                
#define INSERT_PROBLEM "INSERT INTO problem_report (problem_num, project_num, \
                                    severity_num, type_num, status_num, \
                                    submitter_id, title, descr, open_version) \
                        VALUES (nextval ('problem_report_seq'), %d, %d, \
                                %d, %d, getpgusername(), %s, %s, "

#define VERSION_QUERY "SELECT version_num, version_text \
                         FROM project_version \
                        WHERE project_num = %d \
                          AND active = TRUE \
                        ORDER BY order_num "

typedef enum {
   PROJECTS,
   VERSIONS,
   PROBLEM_TYPES,
   SEVERITIES
} option_menu_type;





/*
 *  on_problem_entry_dlg_destroy
 *  
 *  We are done, free any memory that the dialog was using. 
 */
static void 
on_problem_entry_dlg_destroy (GObject  *object,
                              gpointer  user_data)
{
   GtkWidget  *widget;
   GArray     *array;

   widget = lookup_widget (GTK_WIDGET (object), PRJ_MENU);
   g_assert (widget != NULL);
   array = g_object_get_data (G_OBJECT (widget), PK_ARRAY);
   if (array != NULL)
      {
      g_array_free (array, TRUE);
      }

   widget = lookup_widget (GTK_WIDGET (object), VER_MENU);
   g_assert (widget != NULL);
   array = g_object_get_data (G_OBJECT (widget), PK_ARRAY);
   if (array != NULL)
      {
      g_array_free (array, TRUE);
      }

   widget = lookup_widget (GTK_WIDGET (object), PT_MENU);
   g_assert (widget != NULL);
   array = g_object_get_data (G_OBJECT (widget), PK_ARRAY);
   if (array != NULL)
      {
      g_array_free (array, TRUE);
      }
   array = g_object_get_data (G_OBJECT (widget), AUX_DATA_ARRAY);
   if (array != NULL)
      {
      g_array_free (array, TRUE);
      }

   widget = lookup_widget (GTK_WIDGET (object), SVR_MENU);
   g_assert (widget != NULL);
   array = g_object_get_data (G_OBJECT (widget), PK_ARRAY);
   if (array != NULL)
      {
      g_array_free (array, TRUE);
      }

   destroy_dirty_list (object);

   /*
    * If this is a modal dialog, we need to shut down it's message
    * processing loop.
    */
   if (GTK_WINDOW(object)->modal)
      {
      gtk_main_quit ();
      }
}


static void
draw_version_menu (GtkOptionMenu  *prj_menu,
                   GtkWidget      *dlg)
{
   PGconn         *conn;
   PGresult       *res;
   GArray         *prjs;
   GtkOptionMenu  *ver_menu;
   GArray         *ver_pks;
   GString        *sql;
   GtkWidget      *menu;
   GtkWidget      *item;
   gint            prj_num;
   gint            def_ver;
   gint            sel;
   gint            pk;
   gint            i, n;

   prjs = g_object_get_data (G_OBJECT (prj_menu), PK_ARRAY);
   g_assert (prjs != NULL);
   conn = g_object_get_data (G_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   ver_menu = GTK_OPTION_MENU (lookup_widget (dlg, VER_MENU));
   g_assert (ver_menu != NULL);

   /*
    * Get the default version for the currently selected project.
    */
   sel = gtk_option_menu_get_history (prj_menu);
   prj_num = g_array_index (prjs, gint, sel);
   sql = g_string_new ("select open_version from project ");
   g_string_append_printf (sql, "where project_num = %d ", prj_num);
   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Getting default open version")) &&
         !PQgetisnull (res, 0, 0))
      {
      def_ver = atoi (PQgetvalue (res, 0, 0));
      }
   else
      {
      def_ver = 0;
      }

   /*
    * If the ver_menu has any PK array, then we need to destroy
    * it, and the current menu, before building another one.
    */
   ver_pks = g_object_get_data (G_OBJECT (ver_menu), PK_ARRAY);
   if (ver_pks != NULL)
      {
      gtk_option_menu_remove_menu (ver_menu);
      g_array_free (ver_pks, TRUE);
      }

   /*
    * Build the menu, select the default open version if there
    * is one.
    */
   sql = g_string_assign (sql, "select version_num, ");
   sql = g_string_append (sql, "version_text || ' - ' || descr ");
   sql = g_string_append (sql, "from project_version ");
   sql = g_string_append (sql, "where active = TRUE ");
   g_string_append_printf (sql, "and project_num = %d ", prj_num);
   sql = g_string_append (sql, "order by version_num ");
   res = PQexec (conn, sql->str);
   if (chk_sql_error (res, _("Getting version information")))
      {
      sel = 0;
      n = PQntuples (res);
      menu = gtk_menu_new ();
      ver_pks = g_array_sized_new (FALSE, FALSE, sizeof (gint), n);
      for (i = 0; i < n; i++)
         {
         item = gtk_menu_item_new_with_label (PQgetvalue (res, i, 1));
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         gtk_widget_show (item);
         pk = atoi (PQgetvalue (res, i, 0));
         g_array_append_val (ver_pks, pk);
         if (pk == def_ver)
            {
            sel = i;
            }
         }
      gtk_option_menu_set_menu (ver_menu, menu);
      gtk_option_menu_set_history (ver_menu, sel);
      g_object_set_data (G_OBJECT (ver_menu), PK_ARRAY, ver_pks);
      }

   g_string_free (sql, TRUE);
}


/*
 * Draw the Project, Problem type, or Severity option menu.
 */
static void
draw_option_menu (PGconn           *conn, 
                  GtkOptionMenu    *opt_menu,
                  option_menu_type  type)
{
   GtkWidget         *menu;
   GtkWidget         *item;
   PGresult          *res;
   GString           *buffer;
   GString           *errstr;
   GArray            *pks;
   GArray            *aux;
   gint               pk;
   gint               i, n;
   gint               sel;
   gint               defpk;

   buffer = g_string_new ("");
   errstr = g_string_new (_("Filling the "));
   switch (type)
      {
      case PROJECTS:
         g_string_printf (buffer, GET_PROJECTS, PQuser (conn));
         errstr = g_string_append (errstr, _("project menu"));
         break;
      case PROBLEM_TYPES:
         buffer = g_string_assign (buffer, GET_PROBLEM_TYPES);
         errstr = g_string_append (errstr, _("problem type menu"));
         break;
      case SEVERITIES:
         buffer = g_string_assign (buffer, GET_SEVERITIES);
         errstr = g_string_append (errstr, _("severity menu"));
         break;
      default:
         g_assert_not_reached ();
         break;
      }

   /*
    * If there is an old list of PK's, the option menu also has a 
    * menu.  Destroy both of them....
    *
    * Also, if the menu has an aux data array (def severities, etc),
    * destroy it also.
    */
   defpk = INVALID_PK;
   pks = g_object_get_data (G_OBJECT (opt_menu), PK_ARRAY);
   if (pks != NULL)
      {
      sel = gtk_option_menu_get_history (opt_menu);
      defpk = ((sel == -1) ? INVALID_PK : (g_array_index (pks, gint, sel)));
      gtk_option_menu_remove_menu (opt_menu);
      g_array_free (pks, TRUE);
      }
   aux = g_object_get_data (G_OBJECT (opt_menu), AUX_DATA_ARRAY);
   if (aux != NULL)
      {
      g_array_free (aux, TRUE);
      aux = NULL;
      }

   /*
    * Select the data.  Create the PK array, and any AUX array that 
    * may be required.
    */
   res = PQexec (conn, buffer->str);
   if (chk_sql_error (res, errstr->str))
      {
      n = PQntuples (res);
      pks = g_array_sized_new (FALSE, FALSE, sizeof (gint), n);
      if (type == PROBLEM_TYPES)
         {
         aux = g_array_sized_new (FALSE, FALSE, sizeof (gint), n);
         }
      menu = gtk_menu_new ();
      sel = 0;
      for (i = 0; i < n; i++)
         {
         item = gtk_menu_item_new_with_label (PQgetvalue (res, i, NAME_POS));
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         gtk_widget_show (item);
         pk = atoi (PQgetvalue (res, i, PK_NUM_POS));
         g_array_append_val (pks, pk);
         if (pk == defpk)
            {
            sel = i;
            }
         if (type == PROBLEM_TYPES)
            {
            g_assert (aux != NULL);
            pk = atoi (PQgetvalue (res, i, DEF_SEVR_POS));
            g_array_append_val (aux, pk);
            }
         }
      gtk_option_menu_set_menu (opt_menu, menu);
      gtk_option_menu_set_history (opt_menu, sel);
      g_object_set_data (G_OBJECT (opt_menu), PK_ARRAY, pks);
      if (aux != NULL)
         {
         g_object_set_data (G_OBJECT (opt_menu), AUX_DATA_ARRAY, aux);
         }
      }

   PQclear (res);
   g_string_free (buffer, TRUE);
   g_string_free (errstr, TRUE);
}


/*
 * Get the selected problem type.  The aux array associated with the
 * selected problem type will contain the PK of the default severity.
 * Find it and set the severity option menu to it.
 */
static void
set_default_severity (GtkOptionMenu  *pt_menu,
                      GtkOptionMenu  *svr_menu)
{
   gint     sel;
   gint     idx;
   gint     def_sevr;
   GArray  *aux;
   GArray  *pks;

   sel = gtk_option_menu_get_history (pt_menu);
   if (sel >= 0)
      {
      /*
       * For the PT menu, the AUX_DATA_ARRAY contains the 
       * default severity primary keys.
       */
      aux = g_object_get_data (G_OBJECT (pt_menu), AUX_DATA_ARRAY);
      g_assert (aux != NULL);
      def_sevr = g_array_index (aux, gint, sel);
      pks = g_object_get_data (G_OBJECT (svr_menu), PK_ARRAY);
      if (pks != NULL)
         {
         for (idx = 0; idx < pks->len; idx++)
            {
            if (g_array_index (pks, gint, idx) == def_sevr)
               {
               gtk_option_menu_set_history (svr_menu, idx);
               break;
               }
            }
         }
      }
}


/*
 *  on_problem_entry_dlg_show
 *  
 *  First showing the dialog box.  Initialize the project, problem type,
 *  and severity lists.  For the project list, limit the projects to 
 *  just those that the user has access to.
 */
static void 
on_problem_entry_dlg_show (GtkWidget  *widget,
                           gpointer    user_data)
{
   GtkOptionMenu     *prj_menu;
   GtkOptionMenu     *ver_menu;
   GtkOptionMenu     *pt_menu;
   GtkOptionMenu     *svr_menu;
   PGconn            *conn;   

   prj_menu = GTK_OPTION_MENU (lookup_widget (widget, PRJ_MENU));
   ver_menu = GTK_OPTION_MENU (lookup_widget (widget, VER_MENU));
   pt_menu  = GTK_OPTION_MENU (lookup_widget (widget, PT_MENU));
   svr_menu = GTK_OPTION_MENU (lookup_widget (widget, SVR_MENU));
   g_assert (prj_menu != NULL);
   g_assert (ver_menu != NULL);
   g_assert (pt_menu != NULL);
   g_assert (svr_menu != NULL);

   conn = g_object_get_data (G_OBJECT (widget), MY_DB_CONN);
   g_assert (conn != NULL);
   
   draw_option_menu (conn, prj_menu, PROJECTS);
   draw_option_menu (conn, pt_menu, PROBLEM_TYPES);
   draw_option_menu (conn, svr_menu, SEVERITIES);
   draw_version_menu (prj_menu, widget);
   set_default_severity (pt_menu, svr_menu);

   /*
    * Now that the option menus are fully drawn, connect the necessary
    * "changed" handlers.
    */
   g_signal_connect (pt_menu, "changed",
                     G_CALLBACK (set_default_severity), svr_menu);
   g_signal_connect (prj_menu, "changed",
                     G_CALLBACK (draw_version_menu), widget);
}


/*
 *  on_ok_button_clicked
 *  
 *  clicked signal handler for the OK button
 *  
 *  On OK, we want to insert the problem report into the database.  For
 *  the status, we will use the lowest active status.
 */
static void 
on_ok_button_clicked (GtkDialog *dlg)
{
   GtkWidget         *parent;  /* application main window */
   GtkOptionMenu     *prj_menu;
   GtkOptionMenu     *pt_menu;
   GtkOptionMenu     *svr_menu;
   GtkOptionMenu     *ver_menu;
   GtkEntry          *subject_entry;
   GtkTextView       *descr_text;
   GtkTextBuffer     *buff;
   GtkTextIter        start;
   GtkTextIter        end;
   gboolean           ok_to_process = TRUE;
   GString           *subject_str;
   GString           *descr_str;
   GString           *msg_str;
   GString           *buffer;
   GArray            *pks;
   gint               sel;
   gint               pk_proj;
   gint               pk_sevr;
   gint               pk_stat;
   gint               pk_type;
   gint               pk_ver = INVALID_PK;
   gchar             *char_buf;
   PGconn            *conn;
   PGresult          *res;
   GtkWidget         *msg_dlg;
   GConfClient       *client;

   client = g_object_get_data (G_OBJECT (dlg), MY_GCONF_CLIENT);
   g_assert (client != NULL);

   prj_menu = GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (dlg), PRJ_MENU));
   pt_menu = GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (dlg), PT_MENU));
   svr_menu = GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (dlg), SVR_MENU));
   ver_menu = GTK_OPTION_MENU (lookup_widget (GTK_WIDGET (dlg), VER_MENU));
   subject_entry = GTK_ENTRY (lookup_widget (GTK_WIDGET (dlg), SUB_ENTRY));
   descr_text = GTK_TEXT_VIEW (lookup_widget (GTK_WIDGET (dlg), DESCR_TEXT));
   parent = lookup_widget (GTK_WIDGET (dlg), MY_PARENT);
   g_assert (prj_menu      != NULL);
   g_assert (pt_menu       != NULL);
   g_assert (svr_menu      != NULL);
   g_assert (ver_menu      != NULL);
   g_assert (subject_entry != NULL);
   g_assert (descr_text    != NULL);
   g_assert (parent        != NULL);

   conn = g_object_get_data (G_OBJECT (dlg), MY_DB_CONN);
   g_assert (conn != NULL);
   
   subject_str = g_string_new (gtk_entry_get_text (subject_entry));
   g_string_prepare_db_instr (subject_str);
   buff = gtk_text_view_get_buffer (descr_text);
   gtk_text_buffer_get_bounds (buff, &start, &end);
   char_buf = gtk_text_buffer_get_text (buff, &start, &end, FALSE);
   descr_str = g_string_new (char_buf);
   g_free (char_buf);
   g_string_prepare_db_instr (descr_str);

   /*
    * For this one, we must subtract 2 for the extra single quotes we
    * inserted when preparing the string.  Also check for "NULL" if
    * the MIN_SUBJECT_LEN is greater than zero.
    */
   if (ok_to_process && 
         (subject_str->len - 2 < MIN_SUBJECT_LEN || 
          (MIN_SUBJECT_LEN > 0 && !strcmp (subject_str->str, "NULL"))))
      {
      msg_str = g_string_new ("");
      g_string_printf 
         (msg_str, 
          _("You must enter at least %d character(s) in the subject"),
          MIN_SUBJECT_LEN);
      msg_dlg = gtk_message_dialog_new (GTK_WINDOW (dlg), 0,
                                        GTK_MESSAGE_ERROR,
                                        GTK_BUTTONS_OK,
                                        msg_str->str);
      gtk_dialog_run (GTK_DIALOG (msg_dlg));
      gtk_widget_destroy (msg_dlg);
      ok_to_process = FALSE;
      g_string_free (msg_str, TRUE);
      }
   
   /*
    * For this one, we must subtract 2 for the extra single quotes we
    * inserted when preparing the string.  Also check for "NULL" if
    * the MIN_DESCR_LEN is greater than zero.
    */
   if (ok_to_process && 
         (descr_str->len - 2 < MIN_DESCR_LEN ||
          (MIN_DESCR_LEN > 0 && !strcmp (descr_str->str, "NULL"))))
      {
      msg_str = g_string_new ("");
      g_string_printf
         (msg_str, 
          _("You must enter at least %d character(s) in the description"),
          MIN_DESCR_LEN);
      msg_dlg = gtk_message_dialog_new (GTK_WINDOW (dlg), 0,
                                        GTK_MESSAGE_ERROR,
                                        GTK_BUTTONS_OK,
                                        msg_str->str);
      gtk_dialog_run (GTK_DIALOG (msg_dlg));
      gtk_widget_destroy (msg_dlg);
      ok_to_process = FALSE;
      g_string_free (msg_str, TRUE);
      }
   
   /*
    *  If we are OK to process at this point, then let's do the
    *  insert.
    *
    *  The tree selections should have already been checked, so if
    *  there is nothing selected, it is a probramatic issue.  Thus,
    *  the assertion failures...
    */
   if (ok_to_process)
      {
      pks = g_object_get_data (G_OBJECT (prj_menu), PK_ARRAY);
      g_assert (pks != NULL);
      sel = gtk_option_menu_get_history (prj_menu);
      g_assert (sel >= 0);
      pk_proj = g_array_index (pks, gint, sel);

      pks = g_object_get_data (G_OBJECT (ver_menu), PK_ARRAY);
      sel = gtk_option_menu_get_history (ver_menu);
      if (pks != NULL && sel >= 0)
         {
         pk_ver = g_array_index (pks, gint, sel);
	 }

      pks = g_object_get_data (G_OBJECT (pt_menu), PK_ARRAY);
      g_assert (pks != NULL);
      sel = gtk_option_menu_get_history (pt_menu);
      g_assert (sel >= 0);
      pk_type = g_array_index (pks, gint, sel);

      pks = g_object_get_data (G_OBJECT (svr_menu), PK_ARRAY);
      g_assert (pks != NULL);
      sel = gtk_option_menu_get_history (svr_menu);
      g_assert (sel >= 0);
      pk_sevr = g_array_index (pks, gint, sel);

      START_TRANSACTION (conn);
      buffer = g_string_new (GET_LOW_STATUS);
      res = PQexec (conn, buffer->str);
      chk_sql_error (res, _("getting low status"));
      pk_stat = atoi (PQgetvalue (res, 0, 0));
      PQclear (res);

      g_string_printf (buffer, 
                       INSERT_PROBLEM, 
                       pk_proj, pk_sevr, pk_type, pk_stat,
                       subject_str->str, descr_str->str); 

      if (pk_ver == INVALID_PK)
         {
         buffer = g_string_append (buffer, "NULL)");
         }
      else
         {
         g_string_append_printf (buffer, "%d)", pk_ver);
         }
      res = PQexec (conn, buffer->str);
      if (chk_sql_error (res, _("inserting problem report")))
         {
         COMMIT (conn);
         save_dlg_size (client, GTK_WIDGET (dlg), PROB_ENT_SEC);
         gtk_widget_destroy (GTK_WIDGET (dlg));
         refresh_pr_list (parent);
         }
      else
         {
         ROLLBACK (conn);
         }
      PQclear (res);
      g_string_free (buffer, TRUE);
      }

   g_string_free (subject_str, TRUE);
   g_string_free (descr_str, TRUE);
}


/*
 *  on_cancel_button_clicked
 *  
 *  clickec signal handler for the Cancel button
 *  
 *  On Cancel, we just want to close the dialog box.   No insert into
 *  the database.
 */
static void 
on_cancel_button_clicked (GtkDialog *dlg)
{
   GConfClient *client;

   client = g_object_get_data (G_OBJECT (dlg), MY_GCONF_CLIENT);
   g_assert (client != NULL);

   save_dlg_size (client, GTK_WIDGET (dlg), PROB_ENT_SEC);
   gtk_widget_destroy (GTK_WIDGET (dlg));
} 


static void
on_problem_entry_dlg_clicked (GtkDialog *dlg,
                              gint       btn_cd,
                              gpointer   user_data)
{
   switch (btn_cd)
      {
      case GTK_RESPONSE_OK:
         on_ok_button_clicked (dlg);
         break;
      case GTK_RESPONSE_CANCEL:
         on_cancel_button_clicked (dlg);
         break;
      case GTK_RESPONSE_DELETE_EVENT:
         break;
      default:
         g_assert_not_reached();
      }
}


gboolean
problem_entry_dlg_dirty (GtkWidget *dlg)
{
   GtkTextView   *view;
   GtkTextBuffer *buff;

   /*
    * First check the text view widgets, then check the list of other
    * widgets that can make the dialog dirty.
    */
   view = GTK_TEXT_VIEW (lookup_widget (dlg, DESCR_TEXT));
   g_assert (view != NULL);
   buff = gtk_text_view_get_buffer (view);
   if (gtk_text_buffer_get_modified (buff))
      {
      return TRUE;
      }

   return objects_children_are_dirty (G_OBJECT (dlg));
}


GtkWidget* 
create_problem_entry_dlg (GtkWidget *parent,
                          PGconn    *conn)
{
  GtkWidget         *problem_entry_dlg;
  GtkWidget         *dialog_vbox;
  GtkWidget         *vbox;
  GtkWidget         *prjhbox;
  GtkWidget         *verhbox;
  GtkWidget         *pthbox;
  GtkWidget         *svrhbox;
  GtkWidget         *subhbox;
  GtkWidget         *descrhbox;
  GtkWidget         *prjmenu;
  GtkWidget         *vermenu;
  GtkWidget         *ptmenu;
  GtkWidget         *svrmenu;
  GtkWidget         *subentry;
  GtkWidget         *label;
  GtkSizeGroup      *size_group;
  GtkWidget         *scrolledwindow7;
  GtkWidget         *descr_text;
  GConfClient       *client;

  client = g_object_get_data (G_OBJECT (parent), MY_GCONF_CLIENT);
  g_assert (client != NULL);

  problem_entry_dlg = gtk_dialog_new_with_buttons
     (_("New Problem Report"),
      NULL, 0,
      GTK_STOCK_OK,
      GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL,
      GTK_RESPONSE_CANCEL,
      NULL);
  GLADE_HOOKUP_OBJECT_NO_REF (problem_entry_dlg, problem_entry_dlg, 
                              "problem_entry_dlg");
  gtk_container_set_border_width (GTK_CONTAINER (problem_entry_dlg), 5);
  gtk_window_set_resizable (GTK_WINDOW (problem_entry_dlg), TRUE);
  g_signal_connect (problem_entry_dlg, "destroy",
                    G_CALLBACK (on_problem_entry_dlg_destroy), NULL);
  g_signal_connect (problem_entry_dlg, "show",
                    G_CALLBACK (on_problem_entry_dlg_show), NULL);
  g_signal_connect (problem_entry_dlg, "response",
                    G_CALLBACK (on_problem_entry_dlg_clicked), NULL);
  gtk_dialog_set_default_response (GTK_DIALOG (problem_entry_dlg),
                                   GTK_RESPONSE_OK);
  set_dlg_size (client, problem_entry_dlg, PROB_ENT_SEC, 550, 360);

  g_object_set_data (G_OBJECT (problem_entry_dlg), TOP_DOG, problem_entry_dlg);
  g_object_set_data (G_OBJECT (problem_entry_dlg), MY_PARENT, parent);
  g_object_set_data (G_OBJECT (problem_entry_dlg), MY_DB_CONN, conn);
  g_object_set_data (G_OBJECT (problem_entry_dlg), MY_GCONF_CLIENT, client);

  dialog_vbox = GTK_DIALOG (problem_entry_dlg)->vbox;
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, dialog_vbox, "dialog_vbox");
  gtk_box_set_spacing (GTK_BOX (dialog_vbox), MAIN_VBOX_SPACING);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), STD_DLG_BORDER);
  gtk_widget_show (dialog_vbox);

  vbox = gtk_vbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, vbox, "vbox");
  gtk_widget_show (vbox);
  gtk_box_pack_start (GTK_BOX (dialog_vbox), vbox, TRUE, TRUE, 0);

  size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

  /*
   * Project...
   */
  prjhbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, prjhbox, "prjhbox");
  gtk_widget_show (prjhbox);
  gtk_box_pack_start (GTK_BOX (vbox), prjhbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Project:"));
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, label, "prjlbl");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (prjhbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  prjmenu = gtk_option_menu_new ();
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, prjmenu, PRJ_MENU);
  gtk_widget_show (prjmenu);
  gtk_box_pack_start (GTK_BOX (prjhbox), prjmenu, TRUE, TRUE, 0);

  /*
   * Version...
   */
  verhbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, verhbox, "verhbox");
  gtk_widget_show (verhbox);
  gtk_box_pack_start (GTK_BOX (vbox), verhbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Version:"));
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, label, "verlbl");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (verhbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  vermenu = gtk_option_menu_new ();
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, vermenu, VER_MENU);
  gtk_widget_show (vermenu);
  gtk_box_pack_start (GTK_BOX (verhbox), vermenu, TRUE, TRUE, 0);

  /*
   * Problem Type
   */
  pthbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, pthbox, "pthbox");
  gtk_widget_show (pthbox);
  gtk_box_pack_start (GTK_BOX (vbox), pthbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Problem type:"));
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, label, "ptlbl");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (pthbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  ptmenu = gtk_option_menu_new ();
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, ptmenu, PT_MENU);
  gtk_widget_show (ptmenu);
  gtk_box_pack_start (GTK_BOX (pthbox), ptmenu, TRUE, TRUE, 0);

  /*
   * Severity...
   */
  svrhbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, svrhbox, "svrhbox");
  gtk_widget_show (svrhbox);
  gtk_box_pack_start (GTK_BOX (vbox), svrhbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Severity:"));
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, label, "svrlbl");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (svrhbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  svrmenu = gtk_option_menu_new ();
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, svrmenu, SVR_MENU);
  gtk_widget_show (svrmenu);
  gtk_box_pack_start (GTK_BOX (svrhbox), svrmenu, TRUE, TRUE, 0);

  /*
   * Subject...
   */
  subhbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, subhbox, "subhbox");
  gtk_widget_show (subhbox);
  gtk_box_pack_start (GTK_BOX (vbox), subhbox, FALSE, FALSE, 0);

  label = gtk_label_new (_("Subject:"));
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, label, "sublbl");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (subhbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  subentry = gtk_entry_new ();
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, subentry, SUB_ENTRY);
  gtk_widget_show (subentry);
  gtk_box_pack_start (GTK_BOX (subhbox), subentry, TRUE, TRUE, 0);

  /*
   * Description
   */
  descrhbox = gtk_hbox_new (FALSE, STD_BOX_SPACING);
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, descrhbox, "descrhbox");
  gtk_widget_show (descrhbox);
  gtk_box_pack_start (GTK_BOX (vbox), descrhbox, TRUE, TRUE, 0);

  label = gtk_label_new (_("Description:"));
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, label, "descrlbl");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (descrhbox), label, FALSE, FALSE, 0);
  gtk_size_group_add_widget (size_group, label);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);

  scrolledwindow7 = gtk_scrolled_window_new (NULL, NULL);
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, scrolledwindow7, "scrolledwindow7");
  gtk_widget_show (scrolledwindow7);
  gtk_box_pack_start (GTK_BOX (descrhbox), scrolledwindow7, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow7), 
                                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow7),
                                       GTK_SHADOW_IN);

  descr_text = gtk_text_view_new ();
  GLADE_HOOKUP_OBJECT (problem_entry_dlg, descr_text, DESCR_TEXT);
  gtk_widget_show (descr_text);
  gtk_container_add (GTK_CONTAINER (scrolledwindow7), descr_text);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (descr_text), TRUE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (descr_text), GTK_WRAP_WORD);

  return problem_entry_dlg;
}
